import pytest
import re
from a2 import BinaryTree

@pytest.fixture(scope="session", autouse=True)
def example_tree():
    t = BinaryTree('S')
    np = t.add_node('NP')
    vp = t.add_node('VP', child='right')
    t.add_node('John', parent=np)
    v = t.add_node('V', parent=vp)
    t.add_node('ate', parent=v)
    npv = t.add_node('NP', parent=vp, child='right')
    dt = t.add_node('Det', parent=npv)
    t.add_node('a', parent=dt)
    n = t.add_node('N', parent=npv, child='right')
    t.add_node('pizza', parent=n)
    return t

def test_size(example_tree):
    assert len(example_tree.T) == 30

def test_find(example_tree):
    assert set(example_tree.find_node("NP")) == set([1, 6])
    assert set(example_tree.find_node("John")) == set([3])
    assert set(example_tree.find_node("N")) == set([14])
    assert set(example_tree.find_node("pizza")) == set([29])

def test_hight(example_tree):
    assert example_tree.get_height() == 4
    assert example_tree.get_height(1) == 1
    assert example_tree.get_height(2) == 3
    assert example_tree.get_height(3) == 0

def test_depth(example_tree):
    assert example_tree.get_depth(0) == 0
    assert example_tree.get_depth(1) == 1
    assert example_tree.get_depth(2) == 1
    assert example_tree.get_depth(3) == 2

def test_str(example_tree):
    print('-----', example_tree)
    s = str(example_tree)
    s = re.sub('[\r\n\t ]+', ' ', s)
    assert s == '(S (NP John) (VP (V ate) (NP (Det a) (N pizza))))'
